/*
 * (c) Copyright 1999-2011 PaperCut Software International Pty. Ltd.
 */
package biz.papercut.pcng.plugin;

/**
 * The 3rd version of the Topup account on demand plugin add support for querying the balance from the external
 * source (if it's supported).
 * 
 * NOTE: This interface must be used to take full advantage of copier devices like VCC because they need to know
 *       how much credit is available before the copying is performed.
 * 
 * @author matt
 */
public interface TopupAccountOnDemandPlugin3 extends TopupAccountOnDemandPlugin2 {
    
    /**
     * Gets the available balance.
     * 
     * If the plugin wants to limit the maximum amount transfered then a value smaller than their true available
     * balance may be returned.
     * 
     * If the plugin does not have the concept of available credit, then this can be used to set a policy that 
     * determines how much credit is available or whether it's unlimited.
     * 
     * @param username The username we're retrieving the balance for.
     * @param sharedAccount The shared account being used (can be null or an empty string if no shared account used).
     * @param accountCharged The account that will be charged (e.g. the user or the shared account)
     * @return  The available credit.  If the account does not exist the null should be return.  For example, if 
     *              the user doesn't exist, or they are charging to a shared account and the the plugin doesn't 
     *              support shared accounts. NOTE: If there is any other reason why the topup will not occur (e.g. if the topup is disabled
     *              for a class of users), then null should be returned.
     */
    AvailableCredit getAvailableCreditForTopUp(String username, String sharedAccount,
            AccountType accountCharged);
    
    /**
     * Represents the amount of available credit for topup.
     * 
     * @author matt
     */
    class AvailableCredit {
        private final boolean _unlimited;
        private final double _available;
        
        /**
         * @param unlimited True if there is unlimited credit available.
         * @param available If not unlimited the amount of credit available (always >= 0.0).
         */
        public AvailableCredit(boolean unlimited, double available) {
            super();
            _unlimited = unlimited;
            _available = available;
        }

        /**
         * @return the unlimited
         */
        public boolean isUnlimited() {
            return _unlimited;
        }

        /**
         * @return the available
         */
        public double getAvailable() {
            return _available;
        }

        @Override
        public String toString() {
            String str = "unlimited:" + _unlimited + "\navailable: " + _available;
            return str;
        }
    }
}
